## R version should be at least 4
## if not, results based on random numbers will be different

## store current RNG settings
RNGsettings <- RNGkind()
## set RNG settings for sampling
RNGkind(sample.kind = "Rejection")

## ---- load packages ----------------------------------------------------------
require(SOAs)

## --- code for Example 1
## obtain OA(9,4,3,2) directly through code
coef <- SOAs:::ff(3,3)[,2:1] ## fast changing first
   ## remove rows that do not have their first nonzero element 1
   coef <- coef[which(coef[,1]==1 | (coef[,1]==0 & coef[,2]==1)),]
## use coef on the two basic columns
OA9.3.4 <- SOAs:::ff(3,3)%*%t(coef)%%3

## obtain OA(27,13,3,2) directly through code
coef3 <- SOAs:::ff(3,3,3)[,3:1]  ## fast changing first
   ## remove rows that do not have their first nonzero element 1
   coef3 <- coef3[which(coef3[,1]==1 | (coef3[,1]==0 & coef3[,2]==1) | 
                       (coef3[,1]==0 & coef3[,2]==0 & coef3[,3]==1)),]
## use coef on the three basic columns
OA27.3.13 <- SOAs:::ff(3,3,3)%*%t(coef3)%%3

## check that obtaining OA(27,13,3,2) recursively is the same
## stack three R(3,2)
col1to4 <- OA9.3.4[rep(1:9,each=3),]
## obtain e_3(3)
col5 <- rep(0:2,9)
## obtain the matrix and compare to direct construction
col5mat <- matrix(col5, nrow=27, ncol=4)
table(cbind(col1to4, col5, (col1to4 + col5mat)%%3, (col1to4+2*col5mat)%%3) - OA27.3.13)

## ---- code for Examples 6 to 8 / Table 3 ------------------------------------
## separate R file GSOA81strength3.R created the RData
## (but takes a long time for obtaining pat81)
load("GSOA81seed8278.RData")
## has GSOA81 and pat81 and pat27

## extracts the entire Table 3 including the omitted columns
dim_wt_tab(pat81)
     ## argument wtlim=12 omits the omitted columns


## ---- code for Example 9 / Table 4 ------------------------------------------
## csv file must be in the working directory
exampleLHDs <- read.csv2("SunEtAl25runs.csv")

D1 <- exampleLHDs[,1:3]  ## uniform
D2 <- exampleLHDs[,4:6]  ## maximin
D3 <- exampleLHDs[,7:9]  ## maxpro
D4 <- exampleLHDs[,10:12]  ## UPD
colnames(D1) <- colnames(D2) <- colnames(D3) <- colnames(D4) <- paste0("V", 1:3)
rownames(D1) <- rownames(D2) <- rownames(D3) <- rownames(D4) <- 1:25

suntab <- rbind(D1=Spattern(D1, 5, maxwt=NULL),
      D2=Spattern(D2, 5, maxwt=NULL),
      D3=Spattern(D3, 5, maxwt=NULL),
      D4=Spattern(D4, 5, maxwt=NULL))
colnames(suntab) <- paste0("S", 1:6)
suntab


## ---- code for Example 10 ----------------------------------------------------
## L81.3.8 is an OA available in R package DoE.base
round(pat81_el1 <- Spattern(L81.3.8, s=3, maxdim=NULL, maxwt=NULL), 2)


## ---- code for Example 11 / Table 5 ------------------------------------------
## obtain 8-level SOA
SOA_ST <- SOAs_8level(32, m=8, optimize = FALSE)[,c(2:4,8)]

## obtain balanced 16-level GSOA
set.seed(3223)
GSOA_ST <- SOA_ST
for (i in 0:7)
  for (j in 1:4)
  GSOA_ST[SOA_ST[,j]==i,j] <- sample((2*i):(2*i+1))
SOA_ST <- SOAs_8level(32, m=8, optimize = FALSE)[,c(2:4,8)]

## obtain unbalanced 16-level GSOA (not recommended for use)
set.seed(3223)
GSOA_STunbalanced <- SOA_ST
for (i in 0:7)
  for (j in 1:4)
  GSOA_STunbalanced[SOA_ST[,j]==i,j] <- sample(c((2*i):(2*i+1),
                                      rep(sample((2*i):(2*i+1), 1),2)))
     ## using the same order for all pairs within a column
     ## would yield integer contributions,
          ## presumably larger than necessary for early S_5

## obtain 32-level GSOA
set.seed(3223)
GSOA_ST32 <- SOA_ST
for (i in 0:7)
  for (j in 1:4)
  GSOA_ST32[SOA_ST[,j]==i,j] <- sample((4*i):(4*i+3))

## calculate stratification patterns
system.time(pat_SOA_ST <- Spattern(SOA_ST, 2, maxwt=NULL))
## 0.25s
system.time(pat_GSOA_ST <- Spattern(GSOA_ST, 2, maxwt=NULL))
## 3.97s
system.time(pat_GSOA_STunbalanced <- Spattern(GSOA_STunbalanced, 2, maxwt=NULL))
## 3.97s
system.time(pat_GSOA_ST32 <- Spattern(GSOA_ST32, 2, maxwt=12))
## 6.05s  (weights would have gone out to 4*5=20, if not limited to 12)

## obtain dimension by weight tables for them
dim_wt_tab(pat_SOA_ST)
dim_wt_tab(pat_GSOA_ST)
dim_wt_tab(pat_GSOA_STunbalanced)
dim_wt_tab(pat_GSOA_ST32)

## calculate phi_p values
round(phi_p(SOA_ST), 3)
round(phi_p(GSOA_ST), 3)
round(phi_p(GSOA_STunbalanced), 3)
round(phi_p(GSOA_ST32), 3)

## ---- code for Example 12 / Table 6 ------------------------------------------
## obtain a strength 3 level permutation (found by trial and error on seed)
set.seed(93)
OSOA16 <- OSOAs(L16.4.5, el=2)
## obtain stratification patterns with maxwt 6
pat4 <- Spattern(OSOA16, 4, maxwt = 6)
pat2 <- Spattern(OSOA16, 2, maxwt = 6)
## display without the sum column
(dim_wt_tab(pat4))
(dim_wt_tab(pat2))
## calculate phi_p values
## unoptimized
    round(phi_p(OSOAs(L16.4.5,el=2, optimize=FALSE)), 3)
## optimized
    round(phi_p(OSOA16), 3)

RNGkind(sample.kind = RNGsettings[3])  ## reinstate old status
    